IMAGE_NAME = "amazon/aws-cli"
DOCKER_PATH = "docker"
PROG_NAME = "awscliv2"

LINUX_X86_64_URL = "https://awscli.amazonaws.com/awscli-exe-linux-x86_64.zip"
LINUX_ARM_URL = "https://awscli.amazonaws.com/awscli-exe-linux-aarch64.zip"
MACOS_URL = "https://awscli.amazonaws.com/AWSCLIV2.pkg"
